package org.bench4q.master.domain.valueobject.datastatistics;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.bench4q.share.models.agent.BehaviorBriefModel;
import org.bench4q.share.models.agent.BehaviorStatusCodeResultModel;
import org.bench4q.share.models.agent.statistics.DataStatisticsModel;
import org.bench4q.share.models.agent.statistics.AgentBehaviorsBriefModel;
import org.bench4q.share.models.master.statistics.ScriptBehaviorsBriefModel;

public class BehaviorsBriefStatistics extends ScriptStatistics {
	private Map<Integer, Map<Integer, BehaviorStatusCodeResultModel>> map;
	private Map<Integer, String> idUrlMap;

	private Map<Integer, Map<Integer, BehaviorStatusCodeResultModel>> getMap() {
		return map;
	}

	private void setMap(
			Map<Integer, Map<Integer, BehaviorStatusCodeResultModel>> map) {
		this.map = map;
	}

	public Map<Integer, String> getIdUrlMap() {
		return idUrlMap;
	}

	public void setIdUrlMap(Map<Integer, String> idUrlMap) {
		this.idUrlMap = idUrlMap;
	}

	public BehaviorsBriefStatistics() {
		this.setMap(new ConcurrentHashMap<Integer, Map<Integer, BehaviorStatusCodeResultModel>>());
		this.setIdUrlMap(new ConcurrentHashMap<Integer, String>());
	}

	public ScriptBehaviorsBriefModel getStatistics() {
		ScriptBehaviorsBriefModel result = new ScriptBehaviorsBriefModel();
		AgentBehaviorsBriefModel agentBehaviorsBriefModel = new AgentBehaviorsBriefModel();
		List<BehaviorBriefModel> list = new ArrayList<BehaviorBriefModel>();
		for (int behaviorId : this.getMap().keySet()) {
			BehaviorBriefModel behaviorBriefModel = new BehaviorBriefModel();
			behaviorBriefModel.setBehaviorId(behaviorId);
			behaviorBriefModel.setBehaviorUrl(this.getIdUrlMap()
					.get(behaviorId));
			List<BehaviorStatusCodeResultModel> statusList = new ArrayList<BehaviorStatusCodeResultModel>();
			for (int statusCode : this.getMap().get(behaviorId).keySet()) {
				statusList.add(this.getMap().get(behaviorId).get(statusCode)
						.getCopy());
			}
			behaviorBriefModel.setDetailStatusCodeResultModels(statusList);
			this.setBehaviorCount(behaviorBriefModel);
			list.add(behaviorBriefModel);
		}
		agentBehaviorsBriefModel.setBehaviorBriefModels(list);
		result.setBehaviorBriefModels(agentBehaviorsBriefModel
				.getBehaviorBriefModels());
		makeUpResultModelWithSamplingTime(result);
		return result;
	}

	private void setBehaviorCount(BehaviorBriefModel behaviorBriefModel) {
		long totalCount = 0, successfulCount = 0;
		for (BehaviorStatusCodeResultModel behaviorStatusCodeResultModel : behaviorBriefModel
				.getDetailStatusCodeResultModels()) {
			totalCount += behaviorStatusCodeResultModel.getCount();
			if (behaviorStatusCodeResultModel.isSuccess()) {
				successfulCount += behaviorStatusCodeResultModel.getCount();
			}
		}
		behaviorBriefModel.setTotalCount(totalCount);
		behaviorBriefModel.setSuccessfulCount(successfulCount);
	}

	public void add(DataStatisticsModel dataUnit) {
		if (!(dataUnit instanceof AgentBehaviorsBriefModel)) {
			return;
		}
		AgentBehaviorsBriefModel input = (AgentBehaviorsBriefModel) dataUnit;
		for (BehaviorBriefModel behaviorBriefModel : input
				.getBehaviorBriefModels()) {
			guardAllMapsKeyExists(behaviorBriefModel);
			Map<Integer, BehaviorStatusCodeResultModel> behaviorStatusMap = this
					.getMap().get(behaviorBriefModel.getBehaviorId());
			for (BehaviorStatusCodeResultModel newOne : behaviorBriefModel
					.getDetailStatusCodeResultModels()) {
				if (!behaviorStatusMap.containsKey(newOne.getStatusCode())) {
					behaviorStatusMap.put(newOne.getStatusCode(), newOne);
					continue;
				}
				BehaviorStatusCodeResultModel origin = behaviorStatusMap
						.get(newOne.getStatusCode());
				origin.plus(newOne);
			}
		}
	}

	private synchronized void guardAllMapsKeyExists(
			BehaviorBriefModel behaviorBriefModel) {
		if (!this.getMap().containsKey(behaviorBriefModel.getBehaviorId())) {
			this.getMap()
					.put(behaviorBriefModel.getBehaviorId(),
							new ConcurrentHashMap<Integer, BehaviorStatusCodeResultModel>());
		}
		if (!this.getIdUrlMap().containsKey(behaviorBriefModel.getBehaviorId())) {
			this.getIdUrlMap().put(behaviorBriefModel.getBehaviorId(),
					behaviorBriefModel.getBehaviorUrl());
		}
	}
}
